﻿#pragma once

#include <windows.h>
#include <tchar.h>

/**
* @namespace LocalStorage
* 로컬 스토리지에 관련된 클래스의 namespace이다.
*/
namespace tipbase
{
	/**
	* @class ILocalStorageTool
	* LocalStorage의 쓰기, 읽기의 동작을 담당하는 추상 클래스이다.
	*
	* @author : heeyoung.heon
	*/
	class ILocalStorageTool
	{
	public:
		virtual ~ILocalStorageTool() {};

		virtual BOOL IsValid()	= 0;
		virtual BOOL Validate() = 0;

		virtual ULONG Load(LPCTSTR keyName, LPCTSTR subKey, char* pData, ULONG size) = 0;
		virtual ULONG Save(LPCTSTR keyName, LPCTSTR subKey, const char* pData, ULONG size) = 0;

		virtual int  GetLastError() = 0;
		virtual BOOL DeleteKey(LPCTSTR keyName) = 0;
	};

	/**
	* @class LocalStorageManager
	* 로컬스토리지의 저장/로드 툴 인스턴스를 관리하고 동작시키는 클래스이다.
	*
	* @author : heeyoung.heon
	*/
	class LocalStorageManager
	{
		ILocalStorageTool* m_Tool;
		LocalStorageManager();

	public:
		LocalStorageManager(ILocalStorageTool* pTool);

		virtual ~LocalStorageManager();

		/** 
		* @fn BOOL IsValid()
		* @brief 로컬 스토리지가 유효한지 검사하는 함수이다.
		*/
		BOOL IsValid();

		/** 
		* @fn BOOL Validate()
		* @brief 로컬 스토리지를 유효하게 하는 함수이다.
		*/
		BOOL Validate();

		/** 
		* @fn ULONG Load(LPCTSTR keyName, LPCTSTR subKey, char* pData, ULONG size)
		* @brief	로컬 스토리지에 있는 데이터를 로드하는 함수이다.
		* @param
		*			keyName : 데이터의 키이름
		*			subKey  : 데이터의 서브 키 이름
		*					  키에 해당하는 데이터가 있더라도 서브키가 맞지 않으면 데이터를
		*					  로드하지 않는다.
		*					  서브키에 관계없이 키값에 해당하는 데이터를 로드하려면 NULL을 주면 된다.
		*			pData   : 데이터를 가지고 올 버퍼 포인터
		*			bufferSize : 버퍼 길이
		* @return 
		*			로드한 데이터 길이, 0이면 로드 못함
		*/
		ULONG Load(LPCTSTR keyName, LPCTSTR subKey, char* pData, ULONG bufferSize);

		/** 
		* @fn ULONG Save(LPCTSTR keyName, LPCTSTR subKey, const char* pData, ULONG dataSize)
		* @brief	로컬 스토리지에 데이터를 저장하는 함수이다.
		* @param
		*			keyName : 데이터의 키이름
		*			subKey  : 데이터의 서브 키 이름
		*					  서브키를 저장하기 싫으면 NULL을 주면 된다.
		*			pData   : 저장할 데이터 포인터
		*			bufferSize : 버퍼 길이
		* @return 
		*			저장한 데이터 길이, 0이면 저장 못함
		*/
		ULONG Save(LPCTSTR keyName, LPCTSTR subKey, const char* pData, ULONG dataSize);

		/** 
		* @fn BOOL DeleteKey(LPCTSTR keyName)
		* @brief	로컬 스토리지에 있는 키의 데이터를 삭제하는 함수이다.
		* @param
		*			keyName : 데이터의 키이름
		* @return 
		*			성공 / 실패
		*/
		BOOL DeleteKey(LPCTSTR keyName);

		/** 
		* @fn int GetLastError()
		* @brief 마지막 에러를 반환하는 함수이다.
		*/
		int GetLastError();
	};


	/**
	* @class RegistryTool
	* 로컬스토리지의 구현을 레지스트리를 이용하여 구현한 클래스이다.
	*
	* @author : heeyoung.heon
	*/
	class RegistryTool : public ILocalStorageTool
	{
	private:
		HKEY m_RootKey;
		TCHAR m_StoragePath[MAX_PATH];
		LONG m_LastError;
		RegistryTool();

	public:
		RegistryTool(TCHAR* storagePath, HKEY rootKey = HKEY_CURRENT_USER);

		virtual BOOL IsValid();
		virtual BOOL Validate();

		virtual ULONG Load(LPCTSTR keyName, LPCTSTR subKey, char* pData, ULONG size);
		virtual ULONG Save(LPCTSTR keyName, LPCTSTR subKey, const char* pData, ULONG size);

		virtual int  GetLastError();
		virtual BOOL DeleteKey(LPCTSTR keyName);
	};


	class FileTool : public ILocalStorageTool
	{
	private:
		TCHAR m_StoragePath[MAX_PATH];
		LONG m_LastError;
		FileTool();

		ULONG QueryBinaryValue(LPCTSTR filePath, char* pData, ULONG size);
		ULONG SetBinaryValue(LPCTSTR filePath, const char* pData, ULONG size);

	public:
		FileTool(TCHAR* storagePath);

		virtual BOOL IsValid();
		virtual BOOL Validate();

		virtual ULONG Load(LPCTSTR keyName, LPCTSTR subKey, char* pData, ULONG size);
		virtual ULONG Save(LPCTSTR keyName, LPCTSTR subKey, const char* pData, ULONG size);

		virtual int  GetLastError();
		virtual BOOL DeleteKey(LPCTSTR keyName);
	};
};

